@@ -20,6 +20,7 @@ from pyabf.abfHeader import EpochPerDACSection
 from pyabf.abfHeader import DACSection
 from pyabf.abfHeader import ADCSection
 from pyabf.abfHeader import ProtocolSection
+from pyabf.abfHeader import SynchArraySection
 from pyabf.abfHeader import SectionMap
 from pyabf.abfHeader import HeaderV2
 from pyabf.abfHeader import HeaderV1
@@ -173,7 +174,7 @@ class ABF:
         if self._headerV1.sFileCommentNew:
             self.abfFileComment = self._headerV1.sFileCommentNew
         else:
-            self.abfFileComment = self._headerV1.sFileCommentOld    
+            self.abfFileComment = self._headerV1.sFileCommentOld
         self.userList = None
         _tagMult = self._headerV1.fADCSampleInterval / 1e6
         _tagMult = _tagMult / self._headerV1.nADCNumChannels
@@ -252,6 +253,7 @@ class ABF:
         self._stringsIndexed = StringsIndexed(
             self._headerV2, self._protocolSection, self._adcSection,
             self._dacSection, self._stringsSection)
+        self._synchArraySection = SynchArraySection(fb, self._sectionMap)
 
         # create useful variables at the class level
         self.abfVersion = self._headerV2.abfVersionDict
@@ -521,14 +523,6 @@ class ABF:
             with open(self.abfFilePath, 'rb') as fb:
                 self._loadAndScaleData(fb)
 
-        # TODO: prevent re-loading of the same sweep.
-
-        # determine data bounds for that sweep
-        pointStart = self.sweepPointCount*sweepNumber
-        pointEnd = pointStart + self.sweepPointCount
-
-        # start updating class-level variables
-
         # sweep information
         self.sweepNumber = sweepNumber
         self.sweepChannel = channel
@@ -552,11 +546,35 @@ class ABF:
             self.sweepLabelY = "Membrane Potential (mV)"
             self.sweepLabelC = "Applied Current (pA)"
 
+        # determine if this ABF uses variable-length sweeps
+        hasMultipleSweeps = self.sweepCount > 1
+        if hasMultipleSweeps and hasattr(self, "_synchArraySection"):
+            uniqueSweepLengths = set(self._synchArraySection.lLength)
+            isFixedLengthSweeps = len(uniqueSweepLengths) == 1
+        else:
+            isFixedLengthSweeps = True
+
+        # determine data bounds for this sweep
+        if (isFixedLengthSweeps):
+            pointStart = self.sweepPointCount*sweepNumber
+            pointCount = self.sweepPointCount
+        else:
+            pointStart = 0
+            for i in range(1, sweepNumber):
+                pointStart += self._synchArraySection.lLength[i-1]
+            pointCount = self._synchArraySection.lLength[sweepNumber]
+        pointEnd = pointStart + pointCount
+
         # load the actual sweep data
         self.sweepY = self.data[channel, pointStart:pointEnd]
         self.sweepX = np.arange(len(self.sweepY))*self.dataSecPerPoint
         if absoluteTime:
-            self.sweepX += sweepNumber * self.sweepIntervalSec
+            if isFixedLengthSweeps:
+                self.sweepX += sweepNumber * self.sweepIntervalSec
+            else:
+                sweepOffsetPoints = self._synchArraySection.lStart[sweepNumber]
+                sweepOffsetSec = sweepOffsetPoints / self.dataRate
+                self.sweepX += sweepOffsetSec
 
         # default case is disabled
         if not hasattr(self, '_sweepBaselinePoints'):
@@ -572,7 +590,8 @@ class ABF:
             self.sweepY = self.sweepY-blVal
 
         # make sure sweepPointCount is always accurate
-        assert (self.sweepPointCount == len(self.sweepY))
+        if isFixedLengthSweeps:
+            assert (self.sweepPointCount == len(self.sweepY))
 
         # prepare the stimulus waveform table for this sweep/channel
         epochTable = pyabf.waveform.EpochTable(self, channel)
